:
# NAME:
#	regress.sh - setup for DNS regression tests
#
# DESCRIPTION:
#	This module does the setup for 'dns/regress', if we are being
#	run because of a 'cvs commit', we will get at least 2 args, the
#	directory within the repository and the file(s) which are
#	being checked in.  Thus if $PWD/$2 exists, we assume we are
#	running via 'cvs', and since 'cvs' will run us for each
#	sub-dir of $ND_DIR, we want to only loadup named(8) for the first
#	call by 'cvs'.	To do this, we compare our PPID with that
#	stored in the file .scm.pid, if they do not match, then we are
#	the first run of this commit.
#
#	For other SCMs we can simply run status to see what if
#	anything needs checking, and we need not bother with the above.
#
#	For cvs; if we are not the first, we check the exit status of the
#	previous run and if it failed we bail out immediately.
#	Otherwise we set NO_NAMED and continue.
#
#	If NO_NAMED is not set, we startup named(8) on an unofficial
#	port so that we can check that it loads the data happily.
#	Subsequent tests can then query this 'named' to further check
#	the data.  We also arrange to kill the temp 'named' when we are
#	done.
#
#	Because the above would wreak havoc with the live named(8) if
#	we were run in /var/named (or whereever it is), we check that
#	that is not our working directory before doing anything else.
#
#	Because we are reliant on the syslogs from named(8), we check
#	that there is actually LOGS_SPACE free space in the directory
#	that its log lives and that syslog is functioning.
#	We also check for ND_DIR_SPACE free in named(8)'s working
#	directory.  If either test fails we bail out.
#
#	If FORCE_COMMIT is set, then no checks are performed.
#
# BUGS:
#	This would be much simpler if 'named -d' always
#	prevented named(8) from forking, but this is not the case.
#	With bind-9 we can use 'named -f' so can know its pid so that
#	we can kill it when done.
#
#	BIND-9.2 did not dump authoritative data in named_dump.db, so
#	is unsuitable for a lot of our tests.
#	BIND-9.3 and later are ok.
#
# SEE ALSO:
#	debug.sh(1), spacefor.sh(1)
#	
# AUTHOR:
#	Simon J. Gerraty <sjg@crufty.net>

# RCSid:
#	$Id: regress.sh,v 1.44 2023/06/11 18:36:31 sjg Exp $
#
#	@(#) Copyright (c) 1997-2023 Simon J. Gerraty.
#
#	This file is provided in the hope that it will
#	be of use.  There is absolutely NO WARRANTY.
#	Permission to copy, redistribute or otherwise
#	use this file is hereby granted provided that 
#	the above copyright notice and this notice are
#	left intact. 
#      
#	Please send copies of changes and bug-fixes to:
#	sjg@crufty.net
#

for d in $Mydir/../etc/rc_d $RC_D /etc/rc_d
do
    test -s $d/source.sh || continue
    RC_D=`cd $d && 'pwd'`
    break
done
PATH=$Mydir:`cd $Mydir/../bin && 'pwd'`:$PATH:$RC_D
export PATH
. debug.sh

DebugOn regress regress:setup setup

$_ATEXIT_SH . atexit.sh
$_SOURCE_SH . source.sh

echo "$Myname: checking $*"

Exists() {
	[ -f $1 ]
}

ECHO="echo $Myname:"

Error() {
	echo "$Myname: $@" >&2
        Exit 1
}

if [ "$FORCE_COMMIT" ]; then
	echo "$Myname: forcing commit"
	Exit 0
fi
here=`'pwd'`

case "$here" in
*regress.d*)	# regression tests, not for us to check...
	Exit 0;;
*named/*)
	ND_DIR=${ND_DIR:-`echo $here | sed 's,named/.*,named,'`};;
*)
	ND_DIR=${ND_DIR:-$here};;
esac

get_SCM() {
        clues="CVS/Entries .hg .svn .git"
        for clue in $clues
        do
                test -s $ND_DIR/$clue || continue
        	case "$clue" in
		CVS/*) echo cvs; return;;
		.*) echo ${clue#.}; return;;
		esac
	done
}

OS=${OS:-`uname`}
LOCAL=${LOCAL:-/usr/local}
case "$OS" in
NetBSD) LOCALBASE=/usr/pkg;;
esac
LOCALBASE=${LOCALBASE:-$LOCAL}

# provide a simple means of controlling NAMED, BINDVERS etc.
_source .regressrc $ND_DIR
_source .regressrc.$OS $ND_DIR

# we only support bind9 or later
BINDVERS=${BINDVERS:-9}
export BINDVERS
NDCONF=${NAMED_CONF:-$ND_DIR/named.conf}.test
SCM_PIDF=$ND_DIR/.scm.pid
ESTS_FILE=$ND_DIR/.regress.ests
MAKE=${MAKE:-`Which bmake`}
MAKE=${MAKE:-make}

export ND_DIR ESTS_FILE MAKE

# now we want to check that we are NOT running in the live tree
if [ ! -d ${LIVE_DNSDIR:-/dev/null} ]; then
	# avoid doing this more than once,
	REAL_NDCONF=`echo $NDCONF | sed 's/\.test$//'`
	LIVE_DNSDIR=`awk '/^[	]*directory/ { print $2; exit 0 }' $REAL_NDCONF | tr -d '";'`
	if [ -d ${LIVE_DNSDIR:-.} ]; then
		LIVE_DNSDIR=`cd ${LIVE_DNSDIR:-.}; pwd`
	fi
	export LIVE_DNSDIR
fi
if [ x$here = x$LIVE_DNSDIR ]; then
	Error "cannot be run in live DNS directory '$LIVE_DNSDIR'"
fi

# tell rc.sh we want to stop if any test fails
ErrExit=Exit

DebugOn scm_setup
SCM=${SCM:-`get_SCM`}

case "$SCM" in
cvs)
        # if $PWD/$2 is not a directory then $2 must name a file in $PWD.
        # so we assume we were called by cvs.
        if [ -z "$PRE_COMMIT" -a ! -d ${PWD:-$here}/$2 ]; then
	        PRE_COMMIT=yes
                CHECK_FILES="$@"
	fi
        ;;
*)	PRE_COMMIT=yes;;
esac
if [ x"$PRE_COMMIT" != x ]; then
        # check if any files need us...
	if [ -z "$CHECK_FILES" ]; then
	        set -- `scm.sh GIT_STATUS_FLAGS=-s status | sed 's,.* ,,'`
	fi
	case "$1" in
	*/hosts|*/rev|*/mx) need_chk=y;;
	*)
		for f in $*
		do
			[ -f $f ] || continue
			case $f in
			*named*|*.db|*.rev|*.list|*.zones) need_chk=y;;
			esac
		done
		;;
	esac
	if [ ${need_chk:-n} != y ]; then
		echo "$Myname: no need to check '$*'"
		exit 0
	fi
        case "$SCM" in
	cvs)
		# compare PPID with previous (if any)
                case "$PPID" in		# its read-only in some shells
		"") 	PPID=`scanps.sh PID $$ PPID`;;
		esac
        	ppid=`[ -s $SCM_PIDF ] && cat $SCM_PIDF`

        	if [ x$ppid = x$PPID -a x$PPID != x ]; then
        		# we are not the first,
        		# see what the exit status was...
        		ests=`[ -s $ESTS_FILE ] && cat $ESTS_FILE`
        		# if all went well cleanup put a 0 there
        		if [ ${ests:-0} != 0 ]; then
        			Error "already failed"
			fi
        		# ok, we continue, but skip named.
        		NO_NAMED=no
        		export NO_NAMED
		fi
                ;;
	esac
fi

# ensure cleanup processing happens.
DebugOn atexit_setup setup
ExitStatus=1
atexit "ExitStatus=\`cat ${ESTS_FILE:-/dev/null}\`"

# so far we have not touched anything.
# make sure that two people running regress at the same time
# do not trip over each other.
DebugOff scm_setup
DebugOn interlock
Only1Args=none
Only1PidDir=.
. interlock.sh

# if we get to cleanup this will become a 0
echo 1 > $ESTS_FILE
echo $PPID > $SCM_PIDF

DebugOn named_setup

if [ "x$NO_NAMED" = x ]; then

	. getndlog.sh
	
	# if in doubt, set NAMED before hand.
	binddirs=${LOCALBASE}/sbin:${LOCALBASE}/libexec:${LOCAL}/sbin:${LOCAL}/bin:/usr/sbin:/usr/local/etc:/usr/etc
	NAMED=${NAMED:-`Which named $binddirs:$PATH`}

	[ "$NAMED" ] || Error "cannot find named"

	[ -s "$NDCONF" ] || Error "don't know where $NDCONF is"

	case "$BINDVERS" in
	9*)	ND_PORT=${ND_PORT:-53953}
		NDC_PORT=${NDC_PORT:-5953}
		export NDC_PORT
		ND_OPTS="-f -p $ND_PORT -c $NDCONF";;
	esac
	# we call ndlog_def now to find MESSAGES for us.
	ndlog_def
	LOGS=`dirname $MESSAGES`

	# check for space
	spacefor.sh $LOGS ${LOGS_SPACE:-1024} ||
        	Error "Not enough space in $LOGS!"
	# ok, see if syslog is actually working.
	LOGGER=${LOGGER:-logger}
	if [ "$LOGGER" != none ]; then
		$LOGGER -t "$Myname[$$]" -p daemon.notice "syslog is working"
		# 2nd one flushes the first
		$LOGGER -t "$Myname[$$]" -p daemon.notice "syslog is still working"

		for i in 1 1 2 2 2
		do
			sync
			ok=`grep "$Myname\[$$\].*syslog is working" $MESSAGES`
			case "$ok" in
			"")	sleep $i;;
			*)	break;;
			esac
		done
		case "$ok" in
		"")	Error "cannot find my entry in $MESSAGES";;
		esac
	fi
	# Ok, syslog is working lets get to it...
	cd $ND_DIR
	# bind-8 dumps core if it detects a fatal error
	rm -f named.pid core named.core
	spacefor.sh . ${ND_DIR_SPACE:-256} ||
        	Error "Not enough space in $ND_DIR!"
	echo "$Myname: making sure dependencies are up to date"
	$MAKE depend
	$MAKE
	# now named should be happy
	$NAMED $ND_OPTS &
	ND_STARTPID=$!
	# make sure we clean up.
	atexit "kill $ND_STARTPID 2>/dev/null"

	ND_LOG=/tmp/nd$$.log
	ND_DATA=${ND_DATA:-$ND_LOG.data}
	ND_WILDMX=${ND_WILDMX:-$ND_LOG.mx}

	export ND_PID ND_PORT ND_LOG ND_STARTPID ND_DATA ND_WILDMX MESSAGES
	
	if [ -s $MESSAGES ]; then
		trap "Exit 1" 2 3
                
		for i in 1 1 2 4 4 4 4 4 4 4 8 8 8 8
		do
		        if [ -s core -o -s named.core ]; then
			        GetNamedLog 
				exit 1
			fi
			[ -s named.pid ] && break
			kill -0 $ND_STARTPID 2>/dev/null || break
			sleep $i
		done

		if [ -s $ND_DIR/named.pid ]; then
			ND_PID=`cat $ND_DIR/named.pid`
			while kill -0 $ND_PID 2>/dev/null
			do
				sleep 2	       
				GetNamedLog > $ND_LOG
				ok=`egrep 'named.*(answer|exit|running)' $ND_LOG`
				[ "$ok" ] && break
			done
		else
			# named -d only prevents forking if stdin is a tty
			# this mess is to work out the pid...
			for i in 2 4 4 4 4 4 4 4 8 8 8 8
			do
				x=`GetNamedLog | egrep 'named.*(answer|exit|running)' | tail -1`
				case "$x" in
				*exiting*) break;;
				esac
				ND_PID=`echo $x |  sed 's/.*\[\(.*\)\].*/\1/'`
				case "$ND_PID" in
				"") sleep $i;;
				*) break;;
				esac
			done
		fi
		GetNamedLog > $ND_LOG
		[ -s $ND_LOG ] || { echo "$Myname: We appear to have no syslog output!" >&2; exit 1; }
		case "$ND_PID" in
		"") cat $ND_LOG; exit 1;;
		esac
		kill -0 $ND_PID || { cat $ND_LOG; exit 1; }
		# try and ensure we kill it when done
		[ x$ND_PID = x$ND_STARTPID ] || atexit "kill $ND_PID 2>/dev/null"
		[ "$DEBUGGING" ] && cat $ND_LOG
		ND_DUMP=${ND_DUMP:-$ND_DIR/named_dump.db}
		export ND_DUMP
	fi
fi

DebugOff setup named_setup

